require 'perfect_world/error'

module PerfectWorld

  # Writes/Reads data from/to the users clipboard.
  #
  # Works in graphical environments only. So it is not available
  # via SSH and stuff.
  class Clipboard
    class << self

      # Common clipboard tools.
      COMMANDS = [{
        read: 'xclip -selection clipboard -o',
        write: 'xclip -selection clipboard'
      }, {
        read: 'xsel -b -o',
        write: 'xsel -b'
      }, {
        read: 'pbpaste',
        write: 'pbcopy'
      }]

      # Find a working clipboard tool.
      CMD = COMMANDS.find { |c| system("#{c[:read]} &>/dev/null") }

      # Returns true if the clipboard is available, else false.
      def available?
        ! CMD.nil?
      end

      # Writes a string to the clipboard.
      def write(s)
        run do
          IO.popen(CMD[:write], 'w') { |io| io << s }
        end
      end

      alias :<< :write

      # Reads a string from the clipboard.
      def read
        run { `#{CMD[:read]}` }
      end

      private

      # Wraps the system calls to catch errors.
      def run
        raise ClipboardNotAvailable unless available?
        yield
      rescue SystemCallError, IOError => e
        raise Error, e
      end
    end
  end
end
